package SiteStatsTermChanger::CMS;

use strict;
use warnings;

use MT::CMS::Dashboard;
use MT::Stats qw(readied_provider);

sub init_request {
    my ($eh, $app) = @_;

    my $core = MT->component("core");
    $core->registry("applications","cms","widgets","site_stats",
        {
            label    => 'Site Stats',
            template => 'widget/site_stats.tmpl',
            handler  => '$SiteStatsTermChanger::SiteStatsTermChanger::CMS::plugin_site_stats_widget',
            singular => 1,
            set      => 'main',
            view     => [ 'user', 'website', 'blog' ],
            order    => {
                'user'    => 200,
                'website' => 100,
                'blog'    => 100,
            },
            default => 1,
        });
}

sub plugin_site_stats_widget {
    my $app = shift;
    my ( $tmpl, $param ) = @_;

    if ( $param->{blog_id} ) {
        if ( !$app->user->is_superuser ) {
            my $perm = MT::Permission->load(
                {   author_id   => $app->user->id,
                    blog_id     => $param->{blog_id},
                    permissions => { not => "'comment'" }
                }
            );
            $param->{no_permission} = 1
                unless $perm;
        }
    }
    else {
        # Load favorite websites data
        my $websites = MT::CMS::Dashboard::_build_favorite_websites_data( $app,
            { not_count => 1, remove_no_perms => 1 } );
        foreach my $website (@$websites) {
            my $row;
            $row->{id}   = $website->{website_id};
            $row->{name} = $website->{website_name};
            push @{ $param->{object_loop} }, $row;
        }

        # Load favorite blogs data
        my $blogs = MT::CMS::Dashboard::_build_favorite_blogs_data( $app, { not_count => 1 } );
        foreach my $blog (@$blogs) {
            my $row;
            $row->{id}   = $blog->{blog_id};
            $row->{name} = $blog->{blog_name};
            push @{ $param->{object_loop} }, $row;
        }

        my $loop_count
            = defined $param->{object_loop} ? @{ $param->{object_loop} } : 0;
        if ( $loop_count > 1 ) {
            @{ $param->{object_loop} }
                = sort { $a->{name} cmp $b->{name} }
                @{ $param->{object_loop} };
            $param->{blog_id} = $param->{object_loop}->[0]->{id};
        }
        elsif ( $loop_count >= 1 ) {
            $param->{blog_id} = $param->{object_loop}->[0]->{id};
            $param->{name}    = $param->{object_loop}->[0]->{name};
            delete $param->{object_loop};
        }
        else {
            $param->{no_permission} = 1;
        }
    }

    generate_site_stats_data( $app, $param )
        or return
        unless $param->{no_permission};

    $param;
}

sub generate_site_stats_data {
    my $app     = shift;
    my ($param) = @_;
    my $user    = $app->user;
    my $blog_id = $param->{blog_id};
    my $perms   = $app->user->permissions($blog_id)
        or return $app->error( $app->translate("No permissions") );

    my $cache_time = 60 * MT->config('StatsCacheTTL');   # cache for x minutes
    my $stats_static_path = MT::CMS::Dashboard::create_stats_directory( $app, $param ) or return;

    my $file = "data_" . $blog_id . ".json";
    $param->{stat_url} = $stats_static_path . '/' . $file;
    my $path = File::Spec->catfile( $param->{support_path}, $file );

    require MT::FileMgr;
    my $fmgr = MT::FileMgr->new('Local');
    my $time = $fmgr->file_mod_time($path) if -f $path;

    # Get readied provider
    require MT::App::DataAPI;
    my $blog = $app->model('blog')->load($blog_id);
    my $provider = readied_provider( $app, $blog );
    if ($provider) {
        $param->{provider} = $provider;
    }

    # Get Registry
    my $line_settings
        = $app->registry( 'applications', 'cms', 'site_stats_lines' );

    # Judge force purge cache
    my $force_purge_cache = 0;
    my $present_lines     = MT::Request->instance->cache('site_stats_lines');

    my $current_term;
    unless ($present_lines) {
        if ( $fmgr->exists($path) ) {
            my $file = $fmgr->get_data( $path, 'output' );
            $file =~ s/widget_site_stats_draw_graph\((.*)\);/$1/;
            my $data = MT::Util::from_json($file);

            $current_term = $#{$data->{graph_data}};

            $present_lines = $data->{reg_keys} if $data;
            MT::Request->instance->cache( 'site_stats_lines', $present_lines )
                if $present_lines;
        }
    }
    if ($present_lines) {
        my $present_count = @$present_lines;
        my $reg_count     = keys %$line_settings;
        if ( $present_count != $reg_count ) {
            $force_purge_cache = 1;
        }
        else {
            foreach my $reg_key ( keys %$line_settings ) {
                my $match = grep { $_ eq $reg_key } @$present_lines;
                $force_purge_cache = 1 unless $match;
            }
        }
    }

    $force_purge_cache = 1 unless ($current_term + 1) == 60;

    if (   $force_purge_cache
        || ( lc( MT->config('StatsCachePublishing') ) eq 'off' )
        || (   ( lc( MT->config('StatsCachePublishing') ) eq 'onload' )
            && ( !$time || ( time - $time > $cache_time ) ) )
        )
    {
        # Preparing dates of ten days ago.
        require MT::Util;


        my @ten_days_ago_tl
            = MT::Util::offset_time_list( time - ( 60 * 24 * 60 * 60 ),
            $blog_id );

        # Preparing date array
        my @dates;
        for ( my $i = 59; $i >= 0; $i-- ) {
            my @timelist
                = MT::Util::offset_time_list( time - ( $i * 24 * 60 * 60 ),
                $blog_id );
            my $date = sprintf( '%04d-%02d-%02d',
                $timelist[5] + 1900,
                $timelist[4] + 1,
                @timelist[ 3, 2, 1, 0 ] );
            push @dates, $date;
        }

        my @maxes;
        my @counts;
        my @labels;
        my $pv_today;
        my $pv_yesterday;
        my @reg_keys;

        foreach my $key ( keys %$line_settings ) {
            push @reg_keys, $key;

            my $sub          = @counts;
            my $line_setting = $line_settings->{$key};
            if ( my $condition = $line_setting->{condition} ) {
                $condition = MT->handler_to_coderef($condition);
                next unless $condition->( $app, $param );
            }
            my $handler = $line_setting->{handler} || $line_setting->{code};
            $handler = MT->handler_to_coderef($handler);
            if ($handler) {
                $counts[$sub] = $handler->( $app, \@ten_days_ago_tl, $param )
                    or return;
                $maxes[$sub] = 0;
                foreach my $key ( keys %{ $counts[$sub] } ) {
                    $maxes[$sub] = $counts[$sub]->{$key}
                        if $maxes[$sub] < $counts[$sub]->{$key};
                }
                $labels[$sub] = $line_setting->{hlabel}
                    || $app->translate('Not configured');

                if ( $counts[$sub] && $key eq 'count_pageviews' ) {
                    $pv_today     = $counts[$sub]->{ $dates[9] };
                    $pv_yesterday = $counts[$sub]->{ $dates[8] };
                }
            }
        }

        my $max_sub = 0;
        for ( my $i = 0; $i <= $#maxes; $i++ ) {
            $max_sub = $i if $maxes[$max_sub] < $maxes[$i];
        }
        my @rate;
        for ( my $i = 0; $i <= $#maxes; $i++ ) {
            if ( $i == $max_sub ) {
                push @rate, 1;
            }
            else {
                my $rate = $maxes[$i] ? $maxes[$max_sub] / $maxes[$i] : 1;
                push @rate, $rate;
            }
        }

        my $result;
        foreach my $date (@dates) {
            my %row1 = ( x => $date );
            my @row2;
            for ( my $i = 0; $i <= $#counts; $i++ ) {
                my $count
                    = $counts[$i]->{$date}
                    ? $counts[$i]->{$date} * $rate[$i]
                    : 0;
                my $sub = $i ? $i : '';
                $row1{ 'y' . $sub } = $count;

                my %row = (
                    label => $labels[$i],
                    count => defined $counts[$i]->{$date}
                    ? $counts[$i]->{$date}
                    : 0,
                );
                push @row2, \%row;
            }
            push @{ $result->{graph_data} }, \%row1;
            push @{ $result->{hover_data}{data} }, \@row2;
        }
        $result->{pv_today}        = $pv_today;
        $result->{pv_yesterday}    = $pv_yesterday;
        $result->{reg_keys}        = \@reg_keys;
        $result->{can_edit_config} = 1
            if $perms->can_do('edit_config')
            || $perms->can_do('set_publish_paths')
            || $perms->can_do('administer_blog')
            || $perms->can_do('administer');

        $fmgr->put_data(
            'widget_site_stats_draw_graph('
                . MT::Util::to_json($result) . ');',
            $path
        );
    }

    delete $param->{provider};

    1;
}

sub site_stats {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
      var range = {
        unit: 'daily',
        length: 10,
        end: data.graph_data[9].x
      }
HTML
    $old = quotemeta($old);

    $new = <<HTML;
      var range = {
        unit: 'daily',
        length: 60,
        end: data.graph_data[59].x
      }
HTML

    $$template =~ s/$old/$new/;
}

1;
